@@ -1,9 +1,17 @@
 import os, os.path
+import gettext
+DOMAIN = 'gnome-sudoku'
+gettext.bindtextdomain(DOMAIN)
+gettext.textdomain(DOMAIN)
+from gettext import gettext as _
+import gtk.glade
+gtk.glade.bindtextdomain (DOMAIN)
+gtk.glade.textdomain (DOMAIN)
 
-VERSION = "0.4.0"
+VERSION = "0.5.0"
 APPNAME = "GNOME Sudoku"
-COPYRIGHT = 'Copyright (c) 2005, Thomas M. Hinkle. GNU GPL'
-DESCRIPTION = 'GNOME Sudoku is a simple sudoku generator and player.  Sudoku is a japanese logic puzzle.'
+COPYRIGHT = _('Copyright (c) 2005, Thomas M. Hinkle. GNU GPL')
+DESCRIPTION = _('GNOME Sudoku is a simple sudoku generator and player.  Sudoku is a japanese logic puzzle.')
 AUTHORS = ["Thomas M. Hinkle"]
 AUTO_SAVE= True
 MIN_NEW_PUZZLES = 90
@@ -29,5 +37,7 @@ GLADE_DIR = os.path.join(usr,'share','gnome-sudoku')
 BASE_DIR = os.path.join(usr,'share','gnome-sudoku')
 #DATA_DIR = '/var/games/'
 DATA_DIR = os.path.expanduser('~/.gnome2/gnome-sudoku/')
-if not os.path.exists(DATA_DIR): os.makedirs(DATA_DIR)
+
+def initialize_games_dir ():
+    if not os.path.exists(DATA_DIR): os.makedirs(DATA_DIR)
 
