@@ -29,6 +29,14 @@ class GameSelector (gconf_wrapper.GConfWrapper):
         self.dialog.show()
         ret = self.dialog.run()
         self.dialog.hide()
+        return self.handle_response(ret)
+
+    def run_swallowed_dialog (self, swallower):
+        self.setup_dialog()
+        response = swallower.run_dialog(self.dialog)
+        return self.handle_response(response)
+    
+    def handle_response (self, ret):
         if ret==gtk.RESPONSE_OK:
             return self.get_puzzle()
         else:
@@ -77,13 +85,13 @@ class NewGameSelector (GameSelector):
                                  self.sudoku_tracker.sudoku_maker.names[p]
                                  ])
         # now we enumerate some details...
-        for label,prop in [('Squares instantly fillable by filling: ',
+        for label,prop in [(_('Squares instantly fillable by filling: '),
                             d.instant_fill_fillable),
-                           ('Squares instantly fillable by elimination: ',
+                           (_('Squares instantly fillable by elimination: '),
                             d.instant_elimination_fillable),
-                           ('Number of guesses necessary to solve: ',
+                           (_('Number of guesses necessary to solve: '),
                             len(d.guesses)),
-                           ('Difficulty value: ',
+                           (_('Difficulty value: '),
                             d.value),
                            ]:
             self.model.append(itr, [None,None,str(prop),None,0,label])
@@ -96,9 +104,9 @@ class NewGameSelector (GameSelector):
         self.setup_treeview_columns()
         
     def setup_treeview_columns (self):
-        col0 = gtk.TreeViewColumn('Name',gtk.CellRendererText(),text=5)
+        col0 = gtk.TreeViewColumn(_('Name'),gtk.CellRendererText(),text=5)
         col0.set_sort_column_id(5)
-        col1 = gtk.TreeViewColumn("Difficulty",gtk.CellRendererText(),text=2)
+        col1 = gtk.TreeViewColumn(_("Difficulty"),gtk.CellRendererText(),text=2)
         col1.set_sort_column_id(4)
         #col2 = gtk.TreeViewColumn("Detail",gtk.CellRendererText(),text=3)
         #col2.set_sort_column_id(4)
@@ -125,10 +133,10 @@ class OldGameSelector (GameSelector):
 
     def setup_tree (self):
         rend = gtk.CellRendererText()
-        col0 = gtk.TreeViewColumn('Name',rend,text=8)
+        col0 = gtk.TreeViewColumn(_('Name'),rend,text=8)
         col0.set_sort_column_id(8)
         self.tv.append_column(col0)
-        col1=gtk.TreeViewColumn("Difficulty",rend,text=2)
+        col1=gtk.TreeViewColumn(_("Difficulty"),rend,text=2)
         col1.set_sort_column_id(3)
         self.tv.append_column(col1)
         self.tv.insert_column_with_data_func(1, # position
@@ -143,7 +151,7 @@ class OldGameSelector (GameSelector):
                                              self.cell_data_func,
                                              5) # column
         self.tv.get_column(2).set_sort_column_id(5)
-        col2 = gtk.TreeViewColumn("Status",rend,text=6)
+        col2 = gtk.TreeViewColumn(_("Status"),rend,text=6)
         self.tv.append_column(
             col2
             )
@@ -179,7 +187,7 @@ class OldGameSelector (GameSelector):
                 start_time = jar['timer.__absolute_start_time__']
                 finish_time = jar.get('saved_at',time.time())
                 status_val = jar['timer.tot_time']
-                status = "Played for %s"%format_time(jar['timer.tot_time'],round_at=2)
+                status = _("Played for %s")%format_time(jar['timer.tot_time'],round_at=2)
             if not self.sudoku_tracker.sudoku_maker.names.has_key(game):
                 self.sudoku_tracker.sudoku_maker.names[game]=self.sudoku_tracker.sudoku_maker.get_puzzle_name(
                     _('Puzzle')
@@ -203,7 +211,7 @@ class OldGameSelector (GameSelector):
         # within 18 hours, return in form 4 hours 23 minutes ago or some such
         if curtime - val < 18 * 60 * 60:
             cell.set_property('text',
-                              "%s ago"%format_time(curtime-val,round_at=1))
+                              _("%s ago")%format_time(curtime-val,round_at=1))
             return
         tupl=time.localtime(val)
         if curtime - val <  7 * 24 * 60 * 60:
@@ -315,8 +323,8 @@ class HighScores (GameSelector):
         if time_bonus < 1: time_bonus = 1
         score = diff.value * 100 * time_bonus
         score = score - finisher['auto_fills']*10
-        score = score - finisher['impossible_hints']*3
-        score = score - finisher['hints']*3
+        score = score - finisher['impossible_hints']*0.5
+        score = score - finisher['hints']*0.5
         return score
 
     def player_edited_cb (self, renderer, path_string, text):
@@ -417,6 +425,14 @@ class GamePrinter (NewGameSelector):
         self.dialog.connect('response',self.response_cb)
         self.dialog.show()
 
+    def run_swallowed_dialog (self, swallower):
+        # This is broken... tell anyone who calls us not to use it!
+        raise NotImplementedError
+        self.setup_dialog()
+        self.auto_select_puzzles()
+        response = swallower.run_dialog(self.dialog)
+        return self.handle_response(response)
+    
     def response_cb (self, dialog, response):
         if response == gtk.RESPONSE_ACCEPT or response == gtk.RESPONSE_OK:
             mod,rr = self.tv.get_selection().get_selected_rows()
@@ -446,7 +462,7 @@ class GamePrinter (NewGameSelector):
             pass
         elif response==gnomeprint.ui.DIALOG_RESPONSE_PRINT:
             if self.markAsPlayedToggle.get_active():
-                print 'Marking',len(self.sudokus_printed),'sudokus printed'
+                print 'Marking %d sudokus printed'%(len(self.sudokus_printed))
                 for sud,lab in self.sudokus_printed:
                     jar = {}
                     jar['game']=sud.to_string()
@@ -481,7 +497,7 @@ class GamePrinter (NewGameSelector):
         if not possible and not lower and not higher:
             return
         if len(possible) < nsudokus:
-            print 'Only ',len(possible),'puzzles have a difficulty between ',self.min,'and',self.max
+            print 'Only %d puzzles have a difficulty between %d and %d'%(len(possible), self.min, self.max)
             print "We'll have to fudge a bit..."
             choices = possible # start with everything we've got...
             lower.sort(row_sorter)
