@@ -9,7 +9,8 @@ from gettext import gettext as _
 from gettext import ngettext
 from defaults import *
 from timer import ActiveTimer
-from simple_debug import simple_debug
+from simple_debug import simple_debug, options
+from dialog_swallower import SwappableArea
 
 
 icon_factory = gtk.IconFactory()
@@ -25,6 +26,14 @@ gtk.stock_add([('tracks',
                 _('Track moves'),
                 0,0,""),])
 
+try:
+    STOCK_FULLSCREEN = gtk.STOCK_FULLSCREEN
+except:
+    STOCK_FULLSCREEN = _('Full Screen')
+
+SQUARE_TO_GRID_RATIO = 11 # We want our squares to be 1/11th of the space of a grid
+MINIMUM_SQUARE_SIZE = 15
+
 class UI (gconf_wrapper.GConfWrapper):
     ui='''<ui>
     <menubar name="MenuBar">
@@ -57,7 +66,9 @@ class UI (gconf_wrapper.GConfWrapper):
         <separator/>
         <menuitem action="ZoomOnResize"/>
         <separator/>
+        <menuitem action="ToggleNotes"/>
         <menuitem action="ToggleToolbar"/>
+        <menuitem action="ToggleBackground"/>
       </menu>
       <menu action="Game">
          <menuitem action="ShowPossible"/>
@@ -107,12 +118,15 @@ class UI (gconf_wrapper.GConfWrapper):
                      'difficulty':0.0,
                      'minimum_font_size':pango.SCALE * 7, # minimum font-size
                      'minimum_number_of_new_puzzles':MIN_NEW_PUZZLES,
+                     'bg_black':1,
+                     'bg_custom_color':'',
+                     'show_notes':0
                      }    
 
     @simple_debug
     def __init__ (self):
         self.w = gtk.Window()
-        self.w.set_default_size(800,600)
+        self.w.set_default_size(700,675)
         self.timer = ActiveTimer(self.w)
         self.won = False
         gconf_wrapper.GConfWrapper.__init__(self,
@@ -127,101 +141,117 @@ class UI (gconf_wrapper.GConfWrapper):
         self.w.connect('delete-event',self.quit_cb)
         self.vb = gtk.VBox()
         self.uimanager = gtk.UIManager()
-        self.gsd = gsudoku.SudokuGridDisplay()
+        if self.gconf['bg_custom_color']:
+            bgcol = self.gconf['bg_custom_color']
+        elif self.gconf['bg_black']:
+            bgcol = 'black'
+        else:
+            bgcol = None
+        
+        self.gh = gsudoku.GridHull(bgcol); self.gh.show()
+        if bgcol: self.gh.set_bg_color(bgcol)
+        self.gsd = self.gh.gsd
         self.gsd.connect('puzzle-finished',self.you_win_callback)
+        if not self.gconf['show_notes']: self.gsd.hide_notes()
         #self.gsd = gsudoku.SudokuGridDisplay()	 
         self.main_actions = gtk.ActionGroup('MainActions')        
         self.main_actions.add_actions([
-            ('File',None,'_File'),
+            ('File',None,_('_File')),
             ('New',gtk.STOCK_NEW,None,
-             '<Control>n','Generate new game',self.new_cb),
+             '<Control>n',_('Generate new game'),self.new_cb),
             ('Print',gtk.STOCK_PRINT,None,
              None,'Print current game',self.print_game),
             ('PrintMany',gtk.STOCK_PRINT,_('Print _Multiple Sudokus'),
-             None,'Print many sudokus at a time.',self.print_multiple_games),
+             None,_('Print many sudokus at a time.'),self.print_multiple_games),
             #('Quit',gtk.STOCK_QUIT,None,'<Control>q',
             # 'Quit Sudoku game',self.quit_cb),
             ('Close',gtk.STOCK_CLOSE,None,'<Control>w',
-             'Close Sudoku (save game for later)',self.quit_cb),
+             _('Close Sudoku (save game for later)'),self.quit_cb),
             #('Save',gtk.STOCK_SAVE,_('_Save'),
             # '<Control>s','Save game to play later.',
             # self.save_game),
             ('ByHand',gtk.STOCK_EDIT,_('_Enter custom game'),
-             None,'Enter new puzzle by hand (use this to copy a puzzle from another source).',
+             None,_('Enter new puzzle by hand (use this to copy a puzzle from another source).'),
              self.enter_game_by_hand),
             ('Open',gtk.STOCK_OPEN,_('_Resume old game'),
-             '<Control>r','Resume a previous saved game.',
+             '<Control>r',_('Resume a previous saved game.'),
              self.open_game),
-            ('Game',None,'_Game'),
-            ('View',None,'_View'),
-            ('ShowPossible',gtk.STOCK_HELP,'_Hint',
+            ('Game',None,_('_Game')),
+            ('View',None,_('_View')),
+            ('ShowPossible',gtk.STOCK_HELP,_('_Hint'),
              '<Control>i',
-             'Show which numbers could go in the current square.',
+             _('Show which numbers could go in the current square.'),
              self.show_hint_cb),
-            ('AutofillCurrentSquare',gtk.STOCK_APPLY,'_Fill','<Control>f','Automatically fill in the current square if possible.',
+            ('AutofillCurrentSquare',gtk.STOCK_APPLY,_('_Fill'),'<Control>f',_('Automatically fill in the current square if possible.'),
              self.auto_fill_current_square_cb),
-            ('Autofill',gtk.STOCK_REFRESH,'Fill _all squares','<Control>a','Automatically fill in all squares for which there is only one valid value.',
+            ('Autofill',gtk.STOCK_REFRESH,_('Fill _all squares'),'<Control>a',_('Automatically fill in all squares for which there is only one valid value.'),
              self.auto_fill_cb),
-            ('ZoomIn',gtk.STOCK_ZOOM_IN,'_Increase size',
-             '<Control>plus','Increase the size of numbers and squares',
+            ('ZoomIn',gtk.STOCK_ZOOM_IN,_('_Increase size'),
+             '<Control>plus',_('Increase the size of numbers and squares'),
              self.zoom_in_cb),
-            ('ZoomOut',gtk.STOCK_ZOOM_OUT,'_Decrease size',
-             '<Control>minus','Decrease the size of numbers and squares.',
+            ('ZoomOut',gtk.STOCK_ZOOM_OUT,_('_Decrease size'),
+             '<Control>minus',_('Decrease the size of numbers and squares.'),
              self.zoom_out_cb),
-            ('FullScreen',gtk.STOCK_FULLSCREEN,None,
+            ('FullScreen',STOCK_FULLSCREEN,None,
              'F11',None,self.full_screen_cb),
-            ('PuzzleInfo',gtk.STOCK_ABOUT,'Puzzle _Statistics',
-             None,'Show statistics about current puzzle',
+            ('PuzzleInfo',gtk.STOCK_ABOUT,_('Puzzle _Statistics'),
+             None,_('Show statistics about current puzzle'),
              self.show_info_cb),
-            ('Help',None,'_Help',
+            ('Help',None,_('_Help'),
              None,None,None),
             ('About',gtk.STOCK_ABOUT,None,
              None,None,self.show_about),
             ('ShowFaq',gtk.STOCK_HELP,None,
              None,None,self.show_faq),
             ('HighScores',None,_('High _Scores'),
-             None,'Show high scores or replay old games.',
+             None,_('Show high scores or replay old games.'),
              self.show_high_scores_cb),
             ])
         self.main_actions.add_toggle_actions([
             ('AlwaysShowPossible',
              None,
-             '_Always show hint',
+             _('_Always show hint'),
              None,
-             'Always show possible numbers in a square',
+             _('Always show possible numbers in a square'),
              self.auto_hint_cb),
             ('ShowImpossibleImplications',
              None,
-             'Warn about _unfillable squares',
+             _('Warn about _unfillable squares'),
              None,
-             'Warn about squares made unfillable by a move',
+             _('Warn about squares made unfillable by a move'),
              self.impossible_implication_cb),
-            ('Tracker','tracks','_Track additions',
+            ('Tracker','tracks',_('_Track additions'),
              '<Control>T',
-             'Mark new additions in a separate color so you can keep track of them.',
+             _('Mark new additions in a separate color so you can keep track of them.'),
              self.tracker_toggle_cb,False),
-            ('ZoomOnResize',None,'_Adjust size of grid when resizing window',
-             None,'Automatically change the size of numbers and squares to fit the window.',
+            ('ZoomOnResize',None,_('_Adjust size of grid when resizing window'),
+             None,_('Automatically change the size of numbers and squares to fit the window.'),
              ),
-            ('ToggleToolbar',None,'Show _Toolbar',None,None,self.toggle_toolbar_cb,True),
-            ('BackgroundGenerator',None,'Generate new puzzles _while you play',
-             None,'Generate new puzzles in the background while you play. This will automatically pause when the game goes into the background.',
+            ('ToggleToolbar',None,_('Show _Toolbar'),None,None,self.toggle_toolbar_cb,True),
+            ('ToggleNotes',None,_('Show _Notes'),'<Control>O',
+             _('Include room for notes at the top and bottom of squares.'),self.toggle_notes_cb),
+            ('ToggleBackground',None,_('_Black background'),
+             None,_("Background of game is black; otherwise, the background will follow your theme colors."),
+             self.toggle_background_cb,True),
+            ('BackgroundGenerator',None,_('Generate new puzzles _while you play'),
+             None,_('Generate new puzzles in the background while you play. This will automatically pause when the game goes into the background.'),
              self.toggle_generator_cb, True),
             ])
         
         self.edit_actions = gtk.ActionGroup('EditActions')
         self.edit_actions.add_actions(
-            [('Edit',None,'_Edit'),
-             ('Undo',gtk.STOCK_UNDO,'_Undo','<Control>z','Undo last action'),
-             ('Redo',gtk.STOCK_REDO,'_Redo','<Control>y','Redo last action'),
-             ('Clear',gtk.STOCK_CLEAR,'_Clear','<Control>b',"Clear entries you've filled in",self.clear_cb),
+            [('Edit',None,_('_Edit')),
+             ('Undo',gtk.STOCK_UNDO,_('_Undo'),'<Control>z',_('Undo last action')),
+             ('Redo',gtk.STOCK_REDO,_('_Redo'),'<Control>y',_('Redo last action')),
+             ('Clear',gtk.STOCK_CLEAR,_('_Clear'),'<Control>b',_("Clear entries you've filled in"),self.clear_cb),
              # Trackers...
-             ('Tracker%s',None,'No Tracker','<Control>0',None,lambda *args: self.set_tracker(-1)),
-             ('Generator',None,'_Generate new puzzles',None,'Generate new puzzles.',
+             ('Tracker%s',None,_('No Tracker'),'<Control>0',None,lambda *args: self.set_tracker(-1)),
+             ('Generator',None,_('_Generate new puzzles'),None,_('Generate new puzzles.'),
               self.generate_puzzle_gui,),
              ])
         self.edit_actions.add_actions(
-            [('Tracker%s'%n,None,'Tracker _%s'%n,'<Control>%s'%n,None,lambda *args: self.set_tracker(n-1)) for
+            [('Tracker%s'%n,None,_('Tracker _%s')%n, # %s is a number - tracker 1, tracker 2, etc.
+              '<Control>%s'%n,None,lambda *args: self.set_tracker(n-1)) for
              n in range(1,9)])
         self.uimanager.insert_action_group(self.main_actions,0)
         self.uimanager.insert_action_group(self.edit_actions,0)
@@ -242,27 +272,21 @@ class UI (gconf_wrapper.GConfWrapper):
         #self.tb.show()
         self.vb.pack_start(self.tb,fill=False,expand=False)
         self.main_area = gtk.HBox()
-        #align = gtk.Alignment()
-        #align.add(self.main_area)
-        #align.set_padding(18,12,0,18)
-        self.vb.pack_start(self.main_area,True,padding=18)
+        self.swallower = SwappableArea(self.main_area)
+        self.vb.pack_start(self.swallower,True,padding=18)
+        self.swallower.show()
         self.sw_for_gsd = gtk.ScrolledWindow()
         self.sw_for_gsd.set_shadow_type(gtk.SHADOW_NONE)
-        if self.gconf['zoom_on_resize']:
-            self.sw_for_gsd.set_policy(gtk.POLICY_AUTOMATIC,
-                                       gtk.POLICY_AUTOMATIC)
-        else:
-            self.sw_for_gsd.set_policy(gtk.POLICY_NEVER,
-                                       gtk.POLICY_NEVER)
-        rect = self.sw_for_gsd.get_allocation()
-        self.cur_size = rect.width>rect.height and rect.height or rect.width
-        self.sw_for_gsd.connect('size-allocate',self.allocate_cb)
+        self.sw_for_gsd.set_policy(gtk.POLICY_AUTOMATIC,
+                                   gtk.POLICY_AUTOMATIC)
+        #self.sw_for_gsd.set_policy(gtk.POLICY_NEVER,
+        #                           gtk.POLICY_NEVER)
         self.sw_vp = gtk.Viewport()
         self.sw_vp.set_shadow_type(gtk.SHADOW_NONE)
-        self.sw_vp.add(self.gsd)
+        self.sw_vp.add(self.gh)
         self.sw_for_gsd.add(self.sw_vp)
         self.sw_vp.show(); self.sw_for_gsd.show()
-        self.main_area.pack_start(self.sw_for_gsd,expand=True,fill=True,padding=18)
+        self.main_area.pack_start(self.sw_for_gsd,padding=18)
         #self.main_area.pack_start(self.gsd,expand=True,fill=True,padding=18)
         self.hint_label = gtk.Label()
         self.hint_label.show()
@@ -282,7 +306,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.game_box.pack_start(self.by_hand_label,False,)#padding=12)        
         self.by_hand_buttonbox = gtk.HButtonBox()
         self.by_hand_buttonbox.set_spacing(12)
-        self.by_hand_save_button = gtk.Button('_Play game')
+        self.by_hand_save_button = gtk.Button(_('_Play game'))
         self.by_hand_save_button.connect('clicked',self.save_handmade_grid)
         self.by_hand_cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
         self.by_hand_cancel_button.connect('clicked',self.cancel_handmade_grid)
@@ -291,21 +315,14 @@ class UI (gconf_wrapper.GConfWrapper):
         self.game_box.pack_start(self.by_hand_buttonbox,False,padding=18)
         self.game_box.show()
         self.by_hand_widgets = [self.by_hand_label,self.by_hand_buttonbox]
-        #self.main_area.add(galign)
-        #self.main_area.add(self.game_box)
-        #galign = gtk.Alignment()
-        #galign.add(self.game_box)
-        #galign.set_padding(18,12,12,18)
         self.main_area.pack_start(self.game_box,False,padding=12)
-        #self.main_area.pack_start(galign,True,padding=12)
         # Set up trackers
         self.trackers = {}
         self.setup_tracker_interface()
-        #self.game_box.pack_start(self.add_tracker_button,False)
-        #self.sw.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
-        #self.sw.add(self.vb)
-        #self.sw.show()
         self.w.add(self.vb)
+        self.statusbar = gtk.Statusbar(); self.statusbar.show()
+        gobject.timeout_add(500,self.update_statusbar_cb)
+        self.vb.pack_end(self.statusbar,fill=False,expand=False)
         self.worker_connections=[]
         mb.show()
         # sync up toggles with gconf values...
@@ -320,22 +337,23 @@ class UI (gconf_wrapper.GConfWrapper):
               self.main_actions.get_action('BackgroundGenerator')),
              ('show_toolbar',
               self.main_actions.get_action('ToggleToolbar')),
+             ('bg_black',
+              self.main_actions.get_action('ToggleBackground')),
              ('show_tracker',
               self.main_actions.get_action('Tracker')),
+             ('show_notes',
+              self.main_actions.get_action('ToggleNotes')),
              ])
-        
         # set font-size to user-set value...
         self.zoom = self.gconf['font_zoom']
-        if self.zoom:
-            self.gsd.change_font_size(multiplier=self.zoom)
+        if self.zoom and not self.gconf['zoom_on_resize']:
+            self.gh.change_font_size(multiplier=self.zoom)
         else:
             self.zoom = 1
         self.timer.start_timing()
-
         # setup sudoku maker...
         self.sudoku_maker = sudoku_maker.SudokuMaker()
         self.sudoku_tracker = sudoku_maker.SudokuTracker(self.sudoku_maker)
-
         if not self.sudoku_tracker.playing:
             self.main_actions.get_action('Open').set_sensitive(False)
         else:
@@ -356,6 +374,8 @@ class UI (gconf_wrapper.GConfWrapper):
                 #print 'We appear to have lost ',game
                 try:
                     self.gsd.load_game(game)
+                    self.timer.tot_time = game['timer.tot_time']
+                    self.timer.__absolute_start_time__ = game['timer.__absolute_start_time__']
                 except:
                     puz,d=self.sudoku_tracker.get_new_puzzle(self.gconf['difficulty'])
         else:
@@ -369,12 +389,10 @@ class UI (gconf_wrapper.GConfWrapper):
             gobject.timeout_add(1000,lambda *args: self.start_worker_thread() and True)
         self.gsd.show()
         self.w.show()
-        # widgets we want hidden...
-        # self.tracker_ui.hide()
-        # for widg in self.by_hand_widgets: widg.hide()
-        # Set up auto-resizing of font
-        self.just_resized=False        
-        self.resizing = False
+
+        rect = self.sw_for_gsd.get_allocation()
+        self.cur_size = None#rect.height>rect.width and rect.width or rect.height
+        self.sw_for_gsd.connect('size-allocate',self.allocate_cb)
 
     @simple_debug
     def start_worker_thread (self, *args):
@@ -403,65 +421,31 @@ class UI (gconf_wrapper.GConfWrapper):
         self.gconf['difficulty']=self.gconf['difficulty']+0.1
         self.timer.finish_timing()
         self.sudoku_tracker.finish_game(self)
-        #time_string = "%s (%s active)"%(self.timer.total_time_string(),
-        #                                self.timer.active_time_string()
-        #                                )
-        #sublabel = _("You completed the puzzle in %s")%time_string
-        #sublabel += "\n"
-        #sublabel += ngettext("You got %s hint","You got %s hints",self.gsd.hints)%self.gsd.hints
-        #sublabel += "\n"
-        #if self.gsd.impossible_hints:
-        #    sublabel += ngettext("You had %s impossibility pointed out.",
-        #                         "You had %s impossibilities pointed out.",
-        #                         self.gsd.impossible_hints)%self.gsd.impossible_hints
-        #if self.gsd.auto_fills:
-        #    sublabel += ngettext("You used the auto-fill %s time",
-        #                         "You used the auto-fill %s times",
-        #                         self.gsd.auto_fills)%self.gsd.auto_fills
-        #dialog_extras.show_message("You win!",label="You win!",
-        #                           icon=os.path.join(IMAGE_DIR,'winner2.png'),
-        #                           sublabel=sublabel
-        #                           )
         hs = game_selector.HighScores(self.sudoku_tracker)
         hs.highlight_newest=True
-        hs.run_dialog()
+        hs.run_swallowed_dialog(self.swallower)
         self.main_actions.get_action('HighScores').set_sensitive(True)
         self.gsd.blank_grid()
         self.stop_game()
+        self.new_cb()
+        
 
     @simple_debug
     def allocate_cb (self, window, rectangle):
-        rect = window.get_allocation()
-        #print 'font-size=',self.gsd.get_font_size()
-        #print self.cur_size,'->',rect.width,rect.height,
-        short_side = (rect.width > rect.height and rect.height) or rect.width
-        if self.gconf['zoom_on_resize'] and self.cur_size != short_side:
-            #print 'changing'
-            #print 'Short side = ',short_side
-            font_size = short_side * 50
-            if font_size < self.gconf['minimum_font_size']:
-                self.gsd.change_font_size(size=self.gconf['minimum_font_size'])
-                return
-            self.gsd.change_font_size(font_size)
-            #print 'New font size = ',font_size            
+        if hasattr(self,'in_allocate') and self.in_allocate: return
+        self.in_allocate = True
+        if self.gconf['zoom_on_resize']:
+            rect = window.get_allocation()
+            while gtk.events_pending(): gtk.main_iteration()
+            rect = self.sw_for_gsd.get_allocation()
+            gsd_alloc = self.gsd.get_allocation()
+            short_side = (rect.width > rect.height and rect.height) or rect.width
+            square_size = short_side / SQUARE_TO_GRID_RATIO
+            if square_size < MINIMUM_SQUARE_SIZE: square_size = MINIMUM_SQUARE_SIZE
+            if self.cur_size != short_side:
+                self.gsd.change_square_sizes(square_size)
             self.cur_size = short_side
-            while (
-                (
-                (self.sw_for_gsd.get_hscrollbar() and
-                 self.sw_for_gsd.get_hscrollbar().get_property('visible')
-                 )
-                or
-                (self.sw_for_gsd.get_vscrollbar() and
-                 self.sw_for_gsd.get_vscrollbar().get_property('visible')
-                 )
-                ) 
-                and
-                font_size > self.gconf['minimum_font_size']
-                ):
-                #print 'Scrollbar visible - new font size = ',font_size
-                font_size = font_size - (0.5 * pango.SCALE)
-                self.gsd.change_font_size(size=font_size)
-                while gtk.events_pending(): gtk.main_iteration()
+        self.in_allocate = False
 
     @simple_debug
     def initialize_prefs (self):
@@ -475,7 +459,7 @@ class UI (gconf_wrapper.GConfWrapper):
     def new_cb (self,*args):        
         gs = game_selector.NewGameSelector(self.sudoku_tracker)
         gs.difficulty = self.gconf['difficulty']
-        ret =  gs.run_dialog()
+        ret =  gs.run_swallowed_dialog(self.swallower)
         if ret:
             puz,d = ret
             self.gconf['difficulty']=d.value
@@ -504,10 +488,8 @@ class UI (gconf_wrapper.GConfWrapper):
         if not self.won:
             if not self.gsd.grid:
                 self.gconf['current_game']=''
-            elif self.gsd.grid.is_changed():
+            else: #always save the game
                 self.gconf['current_game']=self.sudoku_tracker.save_game(self)
-            else:
-                self.gconf['current_game']=self.gsd.grid.to_string()
         self.stop_worker_thread()
         self.sudoku_tracker.save()
         gtk.main_quit()
@@ -539,7 +521,7 @@ class UI (gconf_wrapper.GConfWrapper):
         #ifi = file(game_file,'r')
         #self.gsd.load_game(ifi.read())
         #ifi.close()
-        puzzl=game_selector.OldGameSelector(self.sudoku_tracker).run_dialog()
+        puzzl=game_selector.OldGameSelector(self.sudoku_tracker).run_swallowed_dialog(self.swallower)
         if puzzl:
             self.stop_game()
             saver.open_game(self,puzzl)
@@ -558,20 +540,19 @@ class UI (gconf_wrapper.GConfWrapper):
                              "game%s"%game_number)
         saver.pickle_game(self, game_loc)
         return game_loc
-    
+
     @simple_debug
     def zoom_in_cb (self,*args):
-        self.just_resized=True
-        gobject.timeout_add(200,lambda *args: setattr(self,'just_resized',False))
-        self.gsd.change_font_size(multiplier=1.1)
-        self.zoom = self.zoom * 1.1        
+        self.cur_size = self.cur_size * 1.1
+        self.gsd.change_square_sizes(self.cur_size/SQUARE_TO_GRID_RATIO)
+        #self.zoom = self.zoom * 1.1        
 
     @simple_debug
     def zoom_out_cb (self,*args):
-        self.just_resized=True
-        gobject.timeout_add(100,lambda *args: setattr(self,'just_resized',False))
-        self.gsd.change_font_size(multiplier=0.9)
-        self.zoom = self.zoom * 0.9
+        self.cur_size = self.cur_size * 0.9
+        self.gsd.change_square_sizes(self.cur_size/SQUARE_TO_GRID_RATIO)
+        #self.gh.change_font_size(multiplier=0.9)
+        #self.zoom = self.zoom * 0.9
 
     def full_screen_cb (self, *args):
         if not hasattr(self,'is_fullscreen'): self.is_fullscreen = False
@@ -658,8 +639,59 @@ class UI (gconf_wrapper.GConfWrapper):
         if widg.get_active(): self.tb.show()
         else: self.tb.hide()
 
+    def toggle_notes_cb (self, widg):
+        if widg.get_active(): self.gsd.show_notes()
+        else: self.gsd.hide_notes()
+
+    def update_statusbar_cb (self, *args):
+        if not self.gsd.grid: return
+        puzz = self.gsd.grid.virgin.to_string()
+        if (not hasattr(self,'current_puzzle_string') or
+            self.current_puzzle_string != puzz):
+            if not self.sudoku_tracker.sudoku_maker.names.has_key(puzz):
+                self.sudoku_tracker.sudoku_maker.names[puzz] = self.sudoku_tracker.sudoku_maker.get_puzzle_name(puzz)
+            self.current_puzzle_string = puzz
+            self.current_puzzle_name = self.sudoku_tracker.sudoku_maker.names[puzz]
+            self.current_puzzle_diff = self.sudoku_tracker.get_difficulty(puzz)
+        tot_string = _("Playing ") + self.current_puzzle_name
+        tot_string += " - " + "%s"%self.current_puzzle_diff.value_string()
+        tot_string += " " + "(%1.2f)"%self.current_puzzle_diff.value
+        if self.timer.tot_time or self.timer.tot_time_complete:
+            time_string = _("%s (%s active)")%(
+                self.timer.total_time_string(),
+                self.timer.active_time_string()
+                )
+            if not self.timer.__timing__:
+                time_string += " %s"%_('paused')
+            tot_string += " - " + time_string
+        if self.gsd.hints and not self.gconf['always_show_hints']:
+            tot_string += " -  " +ngettext("%s hint","%s hints",
+                                   self.gsd.hints)%self.gsd.hints
+        if self.gsd.auto_fills:
+            tot_string += "  " +ngettext("%s auto-fill","%s auto-fills",
+                                    self.gsd.auto_fills)%self.gsd.auto_fills
+        if not hasattr(self,'sbid'):
+            self.sbid = self.statusbar.get_context_id('game_info')
+        self.statusbar.pop(self.sbid)
+        self.statusbar.push(self.sbid,
+                            tot_string)
+        return True
+
+    @simple_debug
+    def toggle_background_cb (self, widg):
+        if widg.get_active():
+            self.gh.set_bg_color('black')
+        else:
+            self.gh.set_bg_color(None)
+
     @simple_debug
     def show_info_cb (self, *args):
+        if not self.gsd.grid:
+            dialog_extras.show_message(parent=self.w,
+                                       title=_("Puzzle Information"),
+                                       label=_("There is no current puzzle.")
+                                       )
+            return
         puzzle = self.gsd.grid.virgin.to_string()
         diff = self.sudoku_tracker.get_difficulty(puzzle)
         information = _("Calculated difficulty: ")
@@ -676,7 +708,7 @@ class UI (gconf_wrapper.GConfWrapper):
         information += str(len(diff.guesses))
         if not self.sudoku_tracker.sudoku_maker.names.has_key(puzzle):
             self.sudoku_tracker.sudoku_maker.names[puzzle]=self.sudoku_tracker.sudoku_maker.get_puzzle_name(
-                'Puzzle')
+                _('Puzzle'))
         name = self.sudoku_tracker.sudoku_maker.names[puzzle]
         dialog_extras.show_message(parent=self.w,
                                    title=_("Puzzle Information"),
@@ -694,6 +726,7 @@ class UI (gconf_wrapper.GConfWrapper):
     @simple_debug
     def show_high_scores_cb (self, *args):
         hs=game_selector.HighScores(self.sudoku_tracker)
+        #replay_game = hs.run_swallowed_dialog(self.swallower)
         replay_game = hs.run_dialog()
         if replay_game:
             self.stop_game()
@@ -718,7 +751,7 @@ class UI (gconf_wrapper.GConfWrapper):
 
     @simple_debug
     def show_faq (self, *args):
-        dialog_extras.show_faq(faq_file=os.path.join(BASE_DIR,'FAQ'))
+        dialog_extras.show_faq(faq_file=os.path.join(BASE_DIR,_('FAQ')))
 
     @simple_debug
     def print_game (self, *args):
@@ -765,7 +798,7 @@ class TrackerBox (gtk.VBox):
         self.tracker_tree.append_column(col2)
         self.tracker_tree.append_column(col1)
         # Our initial row...
-        self.tracker_model.append([-1,None,'No Tracker'])
+        self.tracker_model.append([-1,None,_('No Tracker')])
         self.tracker_tree.get_selection().connect('changed',self.selection_changed_cb)
 
     @simple_debug
@@ -807,7 +840,7 @@ class TrackerBox (gtk.VBox):
         self.tracker_tree.get_selection().select_iter(
             self.tracker_model.append([tracker_id,
                                   pb,
-                                  "Tracker %s"%(tracker_id+1)]
+                                  _("Tracker %s")%(tracker_id+1)]
                                   )
             )
         
@@ -873,16 +906,38 @@ class GamesTracker (sudoku_maker.SudokuTracker):
         
 
 def start_game ():
+    initialize_games_dir()
+    if options.debug: print 'Starting GNOME Sudoku in debug mode'
+    if options.profile:
+        options.profile = False
+        profile_me()
+        return    
     u = UI()
     gobject.threads_init()
     try:
-        gtk.main()
+        #print 'here gooes...'
+        #while gtk.events_pending(): gtk.main_iteration()
+        #print 'allocate!'
+        #u.cur_size=0
+        #u.allocate_cb(u.sw_for_gsd,
+        #              u.sw_for_gsd.get_allocation()
+        #              )
+        print 'main...'
+        gtk.main()        
     except KeyboardInterrupt:
         # properly quit on a keyboard interrupt...
         u.quit_cb()
 
-    
-        
+def profile_me ():
+    print 'Profiling GNOME Sudoku'
+    import tempfile,os.path
+    import hotshot, hotshot.stats
+    pname = os.path.join(tempfile.gettempdir(),'GNOME_SUDOKU_HOTSHOT_PROFILE')
+    prof = hotshot.Profile(pname)
+    prof.runcall(start_game)
+    stats = hotshot.stats.load(pname)
+    stats.strip_dirs()
+    stats.sort_stats('time','calls').print_stats()    
         
 if __name__ == '__main__':
     start_game()
