@@ -1,7 +1,14 @@
 # GUI to Sudoku
+from gettext import gettext as _
 import gtk, sudoku, math, gobject
 import pango, random
 from simple_debug import simple_debug
+from SuperEntry import SuperEntry, get_fontsize_for_y, get_font_height
+
+entry_color = "#fff"
+ro_color = "#ddd"
+
+NO_NOTES_TO_NOTES_RATIO = 2.125
 
 class ParallelDict (dict):
     """A handy new sort of dictionary for tracking conflicts.
@@ -41,13 +48,13 @@ class ParallelDict (dict):
 class Fonts:
     read_only_font = pango.FontDescription()
     read_only_font.set_weight(pango.WEIGHT_ULTRABOLD)
-    read_only_font.set_size(pango.SCALE * 16)
-    read_only_color = "#666"
+    read_only_font.set_size(int(pango.SCALE * 9.25))
+    read_only_color = "#333"
     normal_font = pango.FontDescription()
     normal_font.set_weight(pango.WEIGHT_NORMAL)
-    normal_font.set_size(pango.SCALE * 16)
+    normal_font.set_size(int(pango.SCALE * 9.25))
     error_color = "#FF0000"
-    normal_color = "#000000"
+    normal_color = "#000"
     colors = ["#%02x%02x%02x"%(r,b,g) for r,b,g in [(0,0,255), #blue
                                                     (0,255,0), # green
                                                     (255,0,255),#purple
@@ -66,7 +73,7 @@ def change_font_size (size):
     size = int(size)
     Fonts.normal_font.set_size(size)
     Fonts.read_only_font.set_size(size)
-
+    
 class NumberEntry (gtk.Entry):
     conversions = {10:'A',
                    11:'B',
@@ -76,16 +83,20 @@ class NumberEntry (gtk.Entry):
                    15:'F',
                    16:'G',
                    }
-    def __init__ (self, upper=9):
-        gtk.Entry.__init__(self)
-        for k,v in self.conversions.items(): self.conversions[v]=k
+    def __init__ (self, upper=9, base_color=(entry_color,ro_color)):
         self.color = None
+        self.base_color = base_color[0]
+        self.base_color_RO = base_color[1]
+        self.read_only=False
+        gtk.Entry.__init__(self)
+        if base_color:
+            self.modify_base_color(base_color)
+        for k,v in self.conversions.items(): self.conversions[v]=k        
         self.upper = upper
         self.set_width_chars(2)
         self.set_alignment(0.5)
         self.set_max_length(1)
         self.modify_font(Fonts.normal_font)
-        self.read_only=False
         self.__internal_change__ = False
         self.connect('focus-out-event',self.focus_out_cb)
         self.connect('key-press-event',self.keypress_cb)
@@ -93,6 +104,24 @@ class NumberEntry (gtk.Entry):
         self.unset_color()
         self.show()
 
+    def modify_base_color (self, base_color):
+        if base_color==None:
+            base_color = (None,None)
+        self.base_color,self.base_color_RO = base_color
+        if type(self.base_color)==str:
+            self.base_color=gtk.gdk.color_parse(self.base_color)
+        if type(self.base_color_RO)==str:
+            self.base_color_RO=gtk.gdk.color_parse(self.base_color_RO)
+        else:
+            self.base_color_RO=self.get_style().base[gtk.STATE_INSENSITIVE]
+        if self.read_only:
+            self.modify_bg(gtk.STATE_NORMAL,self.base_color_RO)
+            self.modify_base(gtk.STATE_NORMAL,self.base_color_RO)
+            self.modify_fg(gtk.STATE_NORMAL,self.base_color_RO)
+        else:
+            self.modify_bg(gtk.STATE_NORMAL,self.base_color)
+            self.modify_base(gtk.STATE_NORMAL,self.base_color)
+            self.modify_fg(gtk.STATE_NORMAL,self.base_color) 
 
     def changed_cb (self, *args):
         if self.__internal_change__:
@@ -100,20 +129,23 @@ class NumberEntry (gtk.Entry):
             return True
 
     def set_read_only (self, val):
-        self.set_editable(not val)        
+        self.set_editable(not val)
         #self.set_sensitive(not val)
         self.read_only = val
         if val:
             self.modify_font(Fonts.read_only_font)
+            self.modify_base_color((self.base_color,self.base_color_RO))
             if not self.color: self._set_color_(Fonts.read_only_color)
         else:
             self.modify_font(Fonts.normal_font)
+            self.modify_base_color((self.base_color,self.base_color_RO))
             if not self.color: self._set_color_(Fonts.normal_color)
 
     def _set_color_ (self, color):
         color = self.get_colormap().alloc_color(color)
         self.modify_text(gtk.STATE_NORMAL,color)
 
+    @simple_debug
     def set_color (self, color):
         self.color = color
         self._set_color_(color)
@@ -133,8 +165,10 @@ class NumberEntry (gtk.Entry):
                 self._set_color_(self.color)
             else:
                 self.unset_color()
-        
+
     def set_value (self, val):
+        try: self.x,self.y
+        except: pass
         if val > self.upper:
             raise ValueError("Too large a number!")
         if val >= 10:
@@ -191,22 +225,105 @@ class NumberEntry (gtk.Entry):
         elif name in ['Right','KP_Right']:
             parent.emit('move-focus',gtk.DIR_RIGHT)
             return True
-        
-        
+
+
+class SuperNumberEntry (SuperEntry,NumberEntry):
+
+    # Attributes for which we just provide a proxy to main_entry's
+    # attribute
+    pass_through_attnames = ['x','y']
+
+    def __init__ (self,*args,**kwargs):
+        self.args,self.kwargs = args,kwargs
+        SuperEntry.__init__(self)
+        self.connect = self.main_entry.connect
+        self.set_text = self.main_entry.set_text
+        self.get_text = self.main_entry.get_text
+        self.set_value = self.main_entry.set_value
+        self.get_value = self.main_entry.get_value
+        self.set_error_highlight = self.main_entry.set_error_highlight
+        self.set_color = self.main_entry.set_color
+        self._set_color_ = self.main_entry._set_color_
+        #self.modify_font = self.main_entry.modify_font
+
+    def set_entry_sizes (self, size):
+        if self.top_entry.get_property('visible'):
+            SuperEntry.set_entry_sizes(self,size)
+        else:
+            self.move(self.main_entry,0,0)
+            self.main_entry.set_property('width-request',size)
+            self.main_entry.set_property('height-request',size)
+
+    def __setattr__ (self, attr, val):
+        if attr in pass_through_attnames:
+            return setattr(self.main_entry,attr,val)
+        else:
+            return SuperEntry.__setattr__(self,attr,val)
+
+    def __getattr__ (self, attr):
+        if attr in self.pass_through_attnames:
+            return getattr(self.main_entry,attr)
+        else:
+            try:
+                return SuperEntry.__getattr__(self,attr)
+            except:
+                raise AttributeError, attr
+
+    def hide_notes (self):
+        self.top_entry.hide()
+        self.sub_entry.hide()
+        self.set_entry_sizes(self.size)
+
+    def show_notes (self):
+        self.top_entry.show()
+        self.sub_entry.show()
+        self.set_entry_sizes(self.size)
+
+    def setup_main_entry (self):
+        self.main_entry = NumberEntry(*self.args,**self.kwargs)
+        self.main_entry.set_width_chars(4)
+        self.main_entry.set_alignment(0.5)
+        self.main_entry.set_max_length(1)        
+
+    #def __getattr__ (self, attname):
+    #    # This should raise an attribute error if it fails, which will
+    #    # automatically get us normal behavior
+    #    try:
+    #        print 'getting ',attname,'from',self.main_entry
+    #        return getattr(self.main_entry,attname)
+    #    except:
+    #        print 'fallback'
+    #        raise AttributeError
+
+    def __setattr__ (self, attname, val):
+        if attname=='x': self.main_entry.x = val
+        elif attname=='y': self.main_entry.y = val
+        SuperEntry.__setattr__(self,attname,val)
+
+    def change_square_size (self, side=48, big_font_size=None, small_font_size=None):
+        if self.sub_entry.get_property('visible'):
+            SuperEntry.change_square_size(self,side,
+                                          big_font_size=big_font_size,
+                                          small_font_size=small_font_size)
+        else:
+            if not big_font_size:
+                big_font_size = get_fontsize_for_y(side,self.main_font)
+            SuperEntry.change_square_size(self,side,big_font_size=big_font_size,small_font_size=small_font_size)
+            #self.main_font.set_size(big_font_size)
+            #self.main_entry.modify_font(self.main_font)
+        #self.side = side
+
 class EntryGrid (gtk.Table):
-    def __init__ (self, group_size=9):
-        gtk.Table.__init__(self,rows=group_size,columns=group_size,homogeneous=True,)
-        self.set_row_spacings(4)
-        self.set_col_spacings(4)
-        self.group_size = int(group_size)
-        box_side = int(math.sqrt(self.group_size))
-        for n in range(1,box_side):
-            self.set_row_spacing(box_side*n-1,12)
-            self.set_col_spacing(box_side*n-1,12)
+    
+    def __init__ (self, group_size=9,entry_color=(entry_color,ro_color)):
+        gtk.Table.__init__(self,rows=group_size,columns=group_size,homogeneous=True,
+                           )
+        self.group_size = group_size
+        self.change_spacing(3)
         self.__entries__ = {}
         for x in range(self.group_size):
             for y in range(self.group_size):
-                e = NumberEntry(upper=self.group_size)
+                e = SuperNumberEntry(upper=self.group_size,base_color=entry_color)
                 e.x = x
                 e.y = y
                 self.attach(e,x,x+1,y,y+1,
@@ -216,24 +333,74 @@ class EntryGrid (gtk.Table):
                             ypadding=0)
                 self.__entries__[(x,y)]=e
         self.show_all()
+        self.showing_notes = True
 
     def get_focused_entry (self):
         for e in self.__entries__.values():
             if e.is_focus(): return e
+            else:
+                if True in [ec.is_focus() for ec in e.get_children()]:
+                    return e
 
     def get_font_size (self, size=None):
-        size = Fonts.normal_font.get_size() or pango.SCALE * 12
+        size = Fonts.normal_font.get_size() or (pango.SCALE * 18)
         return size
 
     def change_font_size (self, size=None, multiplier=None):
+        print 'Use of change_font_size deprecated'
+        import traceback; traceback.print_exc()
         if not size and not multiplier:
             raise "No size given you dumbass"
-        elif multiplier:
-            size = multiplier * (Fonts.normal_font.get_size() or pango.SCALE * 12)
-        change_font_size(size)
+        elif size:
+            multiplier = size / float(s)
+        old_side = self.__entries__.values()[0].side
+        new_side = int(old_side * multiplier)
+        print 'side from',old_side,'->',new_side
+        self.change_square_sizes(new_side)
+        print 'new_side ->',self.__entries__.values()[0].side
+
+    def change_square_sizes (self, y):
+        main_size,small_size = None,None
+        self.square_size = y
         for e in self.__entries__.values():
-            if e.read_only: e.modify_font(Fonts.read_only_font)
-            else: e.modify_font(Fonts.normal_font)
+            if not main_size:
+                e.change_square_size(y)
+                main_size = e.main_font.get_size()
+                small_size = e.sub_font.get_size()
+            else:
+                e.change_square_size(y,main_size,small_size)
+            #if e.read_only: e.modify_font(Fonts.read_only_font)
+            #else: e.modify_font(Fonts.normal_font)
+        spacing = int(y/15) or 1
+        self.change_spacing(spacing)
+            
+    def change_spacing (self, small_spacing):
+        self.small_spacing = small_spacing
+        self.big_spacing = small_spacing*3
+        self.set_row_spacings(small_spacing)
+        self.set_col_spacings(small_spacing)
+        box_side = int(math.sqrt(self.group_size))
+        for n in range(1,box_side):
+            self.set_row_spacing(box_side*n-1,self.big_spacing)
+            self.set_col_spacing(box_side*n-1,self.big_spacing)
+
+    def show_notes (self):
+        if self.showing_notes: return
+        self.showing_notes = True
+        #self.change_font_size(multiplier=float(1/NO_NOTES_TO_NOTES_RATIO))
+        for e in self.__entries__.values(): e.show_notes()
+        if hasattr(self,'square_size'):
+            self.change_square_sizes(self.square_size)
+
+    def hide_notes (self):
+        if not self.showing_notes: return
+        self.showing_notes = False
+        #self.change_font_size(multiplier=NO_NOTES_TO_NOTES_RATIO)
+        for e in self.__entries__.values():
+            e.hide_notes()
+        if hasattr(self,'square_size'):
+            self.change_square_sizes(self.square_size)
+            
 
 class SudokuGridDisplay (EntryGrid, gobject.GObject):
 
@@ -281,15 +448,15 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
             vals = list(vals)
             vals.sort()
             if vals:
-                label.set_text("Possible values " + ",".join(self.num_to_str(v) for v in vals))
+                label.set_text(_("Possible values ") + ",".join(self.num_to_str(v) for v in vals))
             elif not entry.get_text():
-                label.set_text("No values are possible!")
+                label.set_text(_("No values are possible!"))
             else:
                 label.set_text("")
 
     @simple_debug
     def num_to_str (self, n):
-        if n >= 10: return NumberEntry.conversions[n]
+        if n >= 10: return SuperNumberEntry.conversions[n]
         else: return str(n)
 
     @simple_debug
@@ -371,7 +538,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         else:
             self.entry_validate(widget)
         if self.show_impossible_implications:
-            self.mark_impossibile_implications(widget.x,widget.y)
+            self.mark_impossible_implications(widget.x,widget.y)
 
     @simple_debug
     def entry_validate (self, widget, *args):
@@ -465,11 +632,13 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         #changed=self.grid.fill_deterministically()
         retval = []
         for coords,val in changed:
-            self.add(coords[0],coords[1],val)
+            self.add(coords[0],coords[1],val)            
             retval.append((coords[0],coords[1],val))
+            if self.show_impossible_implications:
+                self.mark_impossible_implications(*coords)
         if retval: self.auto_fills += 1
         if self.grid.check_for_completeness():
-                self.emit('puzzle-finished')
+            self.emit('puzzle-finished')
         return retval
 
     @simple_debug
@@ -480,7 +649,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
             self.add(filled[0][0],filled[0][1],filled[1])
     
     @simple_debug
-    def mark_impossibile_implications (self, x, y):
+    def mark_impossible_implications (self, x, y):
         implications = self.grid.find_impossible_implications(x,y)
         if implications:
             for x,y in implications:
@@ -492,6 +661,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                 self.__entries__[(x,y)].set_impossible(False)
         self.impossibilities = implications
 
+    @simple_debug
     def create_tracker (self, identifier=0):
         if not identifier: identifier = 0
         while self.trackers.has_key(identifier): identifier+=1
@@ -519,6 +689,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
             Fonts.colors.append("#%02x%02x%02x"%(random.randint(0,255),random.randint(0,255),random.randint(0,255)))
             return self.get_tracker_color(identifier)
 
+    @simple_debug
     def toggle_tracker (self, identifier, value):
         """Toggle tracking for tracker identified by identifier."""
         self.__trackers_tracking__[identifier]=value
@@ -557,24 +728,67 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         if not val: val = self.grid._get_(x,y)
         self.trackers[tracker].remove((x,y,val))
         
+class GridHull (gtk.Alignment):
+    def __init__ (self, bg_color, *args, **kwargs):
+        self.gsd = SudokuGridDisplay(*args,**kwargs)
+        self.eb = gtk.EventBox()
+        gtk.Alignment.__init__(self)
+        self.inner_align = gtk.Alignment()
+        self.eb.add(self.inner_align)
+        self.inner_align.add(self.gsd)
+        self.add(self.eb)
+        self.set_property('xalign',0.5)
+        self.set_property('yalign',0.5)
+        self.set_border_padding(self.gsd.big_spacing)
+        if bg_color: self.set_bg_color(bg_color)
+        self.gsd.show()
+        self.eb.show()
+        self.inner_align.show()
+
+    def set_bg_color (self, color):
+        if type(color)==str:
+            try: color = gtk.gdk.color_parse(color)
+            except: return
+        if not color:
+            self.eb.modify_bg(gtk.STATE_NORMAL,color)
+            for e in self.gsd.__entries__.values():
+                e.modify_base_color(None)
+        else:
+            self.eb.modify_bg(gtk.STATE_NORMAL,color)
+            for e in self.gsd.__entries__.values():
+                e.modify_base_color((entry_color,ro_color))
+
+    def set_border_padding (self, n):
+        self.inner_align.set_padding(n,n,n,n)
+
+    def change_font_size (self, size=None, multiplier=None):
+        self.gsd.change_font_size(size=size,multiplier=multiplier)
+        self.set_border_padding(self.gsd.big_spacing)
 
 if gtk.pygtk_version[1]<8: gobject.type_register(SudokuGridDisplay)
 
 if __name__ == '__main__':
     #eg = EntryGrid()
     size = 9
-    sg = SudokuGridDisplay(
+    af = gtk.AspectFrame(ratio=1,obey_child=False)
+    gh = GridHull(
+        'black',
         grid=sudoku.fiendish_sudoku,
         #grid=sudoku.hard_hex_sudoku,
         group_size=size,
-        )
+        ); gh.show()
+    sg = gh.gsd
     w = gtk.Window()
-    vb = gtk.VBox()
-    vb.add(sg)
-    hint = gtk.Label()
+    vb = gtk.VBox(); vb.show()
+    #vb.add(sg)
+    af.add(gh); af.show()
+    vb.add(af)
+    #vb.pack_start(sw,fill=True,expand=True)
+    hint = gtk.Label(); hint.show()
     vb.add(hint)
-    b = gtk.Button("New Sudoku")
-    db = gtk.SpinButton()
+    b = gtk.ToggleButton("Notes"); b.show()
+    b.set_active(True)
+    db = gtk.SpinButton(); db.show()
     adj=db.get_adjustment()
     adj.lower=0
     adj.upper=25
@@ -599,25 +813,25 @@ if __name__ == '__main__':
         sg.add(4,5,6)
         sg.doing_initial_setup=False
     sg.label = hint
-    b3 = gtk.Button('test')
+    b3 = gtk.Button('test'); b3.show()
     b3.connect('clicked',test_filler_up)
-    b2 = gtk.Button('Blank')
-    b2.connect('clicked',lambda *args: sg.blank_grid())
-    b.connect('clicked',new_sudoku)
+    def toggle_notes (but,*args):
+        if but.get_active(): sg.show_notes()
+        else: sg.hide_notes()
+    b.connect('clicked',toggle_notes); b.show()
     w.add(vb)
     vb.add(db)
     vb.add(b)
-    vb.add(b2)
     vb.add(b3)
     #sg.change_font_size(multiplier=3)
-    hb = gtk.HBox()
+    hb = gtk.HBox(); hb.show()
     vb.add(hb)
-    zib = gtk.Button(stock=gtk.STOCK_ZOOM_IN)
+    zib = gtk.Button(stock=gtk.STOCK_ZOOM_IN); zib.show()
     zib.connect('clicked',lambda *args: sg.change_font_size(multiplier=1.1))
-    zob = gtk.Button(stock=gtk.STOCK_ZOOM_OUT)
+    zob = gtk.Button(stock=gtk.STOCK_ZOOM_OUT); zob.show()
     zob.connect('clicked',lambda *args: sg.change_font_size(multiplier=0.9))
     hb.add(zib)
     hb.add(zob)
-    w.show_all()
+    w.show()
     w.connect('delete-event',lambda *args: gtk.main_quit())
     gtk.main()
