@@ -213,7 +213,7 @@ class SudokuPrinter:
             dialog.hide()            
         elif response == gnomeprint.ui.DIALOG_RESPONSE_PREVIEW:
             if not self.drawn: self.draw_sudokus()
-            w=gnomeprint.ui.JobPreview(self.job,'Print Preview')
+            w=gnomeprint.ui.JobPreview(self.job,_('Print Preview'))
             w.set_transient_for(dialog)
             #w.present()
             w.set_property('allow-grow',1)
