@@ -239,15 +239,12 @@ class SudokuSolver (SudokuGrid):
             
     def auto_fill (self):
         changed = []
-        try: change = self.fill_must_fills()
+        try: changed = self.fill_must_fills()
         except UnsolvablePuzzle: return changed
-        while change:
-            changed.extend(change)
-            try: change = self.fill_must_fills()
-            except: return changed
-            try: change.extend(self.fill_deterministically())
-            except: return changed + change
-        return changed
+        try:
+            changed.extend(self.fill_deterministically())
+        finally:
+            return changed
 
     def fill_must_fills (self):
         changed = []
@@ -672,11 +669,11 @@ class DifficultyRating:
             print name,': ',stat
 
     def value_string (self):
-        if self.value > self.very_hard_range[0]: return self.very_hard
-        elif self.value > self.hard_range[0]: return self.hard
-        elif self.value > self.medium_range[0]: return self.medium
+        if self.value > self.very_hard_range[0]: return _(self.very_hard)
+        elif self.value > self.hard_range[0]: return _(self.hard)
+        elif self.value > self.medium_range[0]: return _(self.medium)
         #elif self.value > 0.3: return "Medium"
-        else: return self.easy
+        else: return _(self.easy)
 
 class SudokuRater (SudokuSolver):
 
