@@ -321,7 +321,7 @@ class SudokuMaker:
                 self.all_puzzles[p]=d
                 self.solutions_by_puzzle[p]=solution
                 if not self.names.has_key(p):
-                    self.names[p]=self.get_puzzle_name('Puzzle')
+                    self.names[p]=self.get_puzzle_name(_('Puzzle'))
 
 
         
@@ -430,7 +430,7 @@ class SudokuMaker:
                 closest = diff
         return ret
 
-    def get_puzzle_name (self, base_name="Puzzle"):
+    def get_puzzle_name (self, base_name=_("Puzzle")):
         if not self.top_name.has_key(base_name):
             self.top_name[base_name]=1
         n=self.top_name[base_name]
