@@ -96,7 +96,7 @@ class gnome_sudoku_install(install):
     
     def initialize_options(self):
         install.initialize_options(self)
-        self.prefix = '/usr/local'
+        self.prefix = '/usr'
         # if self.sysconfdir is not a absolute path it will
         # be prefixed by self.prefix
         self.sysconfdir = 'etc'
